#!/usr/bin/env python3

__author__ = "Ahmed Hermas"
__copyright__ = "Copyright 2021, © Digified "
__license__ = "MIT"
__version__ = "0.1.0"
__email__ = "a7medhermas@gmail.com"

from face_utils.detection.detectors import Detector
